module snag.core.rules;

import snag.config;
import snag.core.exception;
import std.algorithm;
import std.array;
import std.path;
import std.stdio;
import std.string;
import std.conv;
import std.container;
import std.process;

class SnagRules {
	private string[] _rules;
	private SnagConfig _config;
	private string[] _baseCommand;

	private string[] generateGitignoreRules(string finalPath) {
		string[] rules;
		string[] parts = finalPath.split("/").filter!(p => !p.empty).array;

		if (parts.length == 0) return rules;

		rules ~= "/*";
		rules ~= "!/" ~ parts[0];
		rules ~= "/" ~ parts[0] ~ "/*";

		if (parts.length > 1) {
			string currentPath = "/" ~ parts[0];
			foreach (i; 1 .. parts.length) {
				currentPath ~= "/" ~ parts[i];
				rules ~= "!" ~ currentPath;
				if (i < parts.length.to!int - 1) {
					rules ~= currentPath ~ "/*";
				}
			}
		}

		return rules;
	}

	private void generate() {
		string[] rules;
		auto tempRules = new RedBlackTree!string;

		_config.tracking.each!(
			track => rules ~= generateGitignoreRules(track)
		);

		rules.each!((rule) {
			if (rule in tempRules) return;
			tempRules.insert(rule);
			_rules ~= rule;
		});

		_rules ~= _config.ignore;
	}

	this(SnagConfig config) {
		_config = config;

		_baseCommand = format(
			"git --git-dir=%s --work-tree=%s",
			config.git, config.project
		).split();
	}

	void create() {
		auto result = execute(_baseCommand ~ ["rev-parse", "--git-dir"]);
		result.status &&
			throw new SnagException(
				"A problem occurred while checking the repository: "
				~ result.output.strip('\n')
			);

		generate();

		string gitignore = _config.git.buildPath("info/exclude");

		File file = File(gitignore, "w");

		_rules.each!(rule => file.writeln(rule));
	}

	// git ls-files -i -c --exclude-standard -z | xargs -0 git rm --cached

	// git ls-files -i -c --exclude-standard
	// git rm --cached <path-to-file>
}
